@@ -74,13 +74,6 @@
 
 #define NO_TYPE 999
 
-#define CBC 0
-#define ECB 1
-#define CFB 2
-#define OFB 3
-
-#define MAX_ALGO 11
-
 #define ENCRYPT 0
 #define DECRYPT 1
 
