@@ -22,9 +22,6 @@
 #endif
 
 #include <xmalloc.h>
-
-#define MAX_KEY_LEN 300
-
 #include <lcrypt.h>
 #include <bzero.h>
 #include "extra.h"
@@ -331,7 +328,9 @@ struct passwd* pwd;
 
   pwd=getpwuid(uid);
   home=malloc(strlen(pwd->pw_dir)+strlen(cfile)+2);
+ if (pwd!=NULL) {
   strcpy(home, pwd->pw_dir);
+ }
   strcat(home, "/");
   strcat(home, cfile);
   
