@@ -1,8 +1,12 @@
+#define MAX_KEY_LEN 300
+
 char * get_password(int numofchar,int bitmode,int mode,int type, unsigned int *len);
 int show_mode(int ende, int stype,int smode,char *output,int bitmode);
 char** read_key_file(char * file, int* num);
 char *get_cfile(int uid, char*);
 
+char* get_modes_name(int mode);
+
 char *my_getpass();
 
 int check_file_head(char * filename, unsigned int *algorithm, unsigned int *mode, unsigned int *bit7_flag,unsigned int stream_flag);
