@@ -42,13 +42,13 @@ int keysize;
     tmpkey=xmalloc(*len);
   }
   else {
-    tmpkey=xmalloc(128);
+    tmpkey=xmalloc(MAX_KEY_LEN);
   }
 
   keyword=xcalloc(1, get_key_size(algorithm));
 
 
-  if (key==NULL) 	{
+  if (key==NULL) {
     tmp=get_password( keysize, bit7_flag, enc_mode, algorithm , len);
     if (tmp==NULL) return NULL;
     *len=strlen(tmp);
