@@ -74,9 +74,9 @@ void usage(char *name)
  -a --algorithm ALGORITHM     specify the encryption and decryption algorithm.\n\
                               Legal parameters are:\n"));
 
-   for (tmpi=0; tmpi<(MAX_ALGO+1); tmpi++) {
-    tmpc=get_algorithms_name(tmpi);
-    if (tmpc!=NULL) {
+   for (tmpi=0; tmpi<255; tmpi++) {
+    if (is_ok_algorithm(tmpi)==0) {
+      tmpc=get_algorithms_name(tmpi);
       fprintf(stdout, "                              - %s\n", tmpc);
       free(tmpc);
     }
@@ -138,7 +138,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 void mcrypt_version() {
 
   fprintf (stderr, _("Mini-crypt v.%s (%s-%s-%s)\n"), VERSION, T_CPU, T_VENDOR, T_OS);
-  fprintf (stderr, _("Copyright (C) 1998,1999 Nikos Mavroyanopoulos (nmav@i-net.paiko.gr)\n"));
+  fprintf (stderr, _("Copyright (C) 1998,1999 Nikos Mavroyanopoulos <nmav@i-net.paiko.gr>\n"));
 
 }
 
@@ -262,7 +262,7 @@ int main(int argc,char **argv)
           din=info.din;
           ein=info.ein;
         }
-
+          
         if (info.real_random_flag==TRUE) {
 #ifdef HAVE_DEV_RANDOM
           real_random_flag=TRUE;
@@ -390,7 +390,7 @@ for(i=0;i<file_count;i++) {
     if (kin==2 && i<=(keys-1)) {
        keyword=keyfile[i];
        if (i!=0) { 
-         Bzero(keyfile[i-1],strlen(keyfile[i-1])); 
+         Bzero(keyfile[i-1], strlen(keyfile[i-1])); 
          free(keyfile[i-1]); /* Free previous keyword */
        }
     }
@@ -534,7 +534,7 @@ for(i=0;i<file_count;i++) {
              }
            }
 
-                      if (mode<=MAX_MODE) {
+                      if (is_ok_mode(mode)==0) {
 
   			   switch (type) {
 
@@ -542,12 +542,17 @@ for(i=0;i<file_count;i++) {
 				 x=encrypt_ucrypt(dinfile,outfile,keyword);
 				 break;			 
 
+				case DES_COMPAT:
+				 bare_flag=TRUE;
+				 x=decrypt_desc(dinfile,outfile,keyword);
+				 break;			 
+
                                 case NO_TYPE:
                                  x=1;
                                  break;
 
                                 default:
-                                 if (type<=MAX_ALGO) {
+                                 if (is_ok_algorithm(type)==0) {
                                    x=decrypt_general(type,dinfile,outfile,keyword);
                                  }
                                  else {
@@ -603,15 +608,20 @@ for(i=0;i<file_count;i++) {
 
 /* Encrypt */
 	if (ein==TRUE) {
-                if (mode<=MAX_MODE) {
+                if (is_ok_mode(mode)==0) {
 
 		      switch (type) {
 	        	case CRYPT:
 	        	 x=encrypt_ucrypt(einfile,outfile,keyword);
 	        	 break;
 	        	 
+	        	case DES_COMPAT:
+	        	 bare_flag=TRUE;
+	        	 x=encrypt_desc(einfile,outfile,keyword);
+	        	 break;
+	        	 
 	        	default:
- 	        	 if (type<=MAX_ALGO) {
+ 	        	 if (is_ok_algorithm(type)==0) {
  	        	   x=encrypt_general(type,einfile,outfile,keyword);
  	        	 }
  	        	 else {
@@ -661,11 +671,16 @@ for(i=0;i<file_count;i++) {
 
 	                }
 	                
+
 	                return_val+=x;
 		}
 
    } /* the main loop */
 
+/* Clear the last keyword used */
+   if (keyword!=NULL) Bzero(keyword, strlen(keyword));
+   free(keyword);
+
 /* Clear ALL arguments in the command line */
   for (i=0; i<argc; i++) {
     Bzero(argv[i],strlen(argv[i]));
@@ -853,11 +868,7 @@ char *command=NULL;
 
 word32 *IV=0;
 word32 *ciphertext=xmalloc(blocksize);
-FILE * TOF; 
-
- TOF=tmpfile();
- write_lock( fileno((FILE*)TOF));
-        
+FILE * TOF=NULL; 
 
 if (mode==CBC || mode==CFB || mode==OFB) {
  IV=xmalloc(blocksize);
@@ -916,12 +927,19 @@ if (mode==CBC || mode==CFB || mode==OFB) {
 	  RTOF=fopen(tofile,"wb");
 	  if (RTOF==NULL) {perror("fopen");return -1;}
 	  if (write_lock( fileno((FILE*)TOF))==-1) return -1;
-    
-    }
+        }
 #endif
   }
 
 
+if (bare_flag==FALSE) {
+ TOF=tmpfile();
+ write_lock( fileno((FILE*)TOF));
+}
+else {
+ TOF=RTOF;
+}
+                                                                        
 
   if (bare_flag==FALSE) {
     write_file_head( fileno((FILE*)TOF), type, mode, bit7_flag);
@@ -1153,19 +1171,21 @@ switch (mode) {
    fseek(TOF,6,SEEK_SET);
    if (fwrite(ciphertext, 1, blocksize, TOF) == 0) {perror("fwrite"); return 1;}
 
- }
+ } /* if bare_flag==FALSE */
 
 /* Copy the temporary to RTOF */
 
- fseek(TOF,0,SEEK_SET);
- copychar=xmalloc(4096);
+  if (bare_flag==FALSE) {
+    fseek(TOF,0,SEEK_SET);
+    copychar=xmalloc(4096);
 
- do {
-    how=fread(copychar, 1, 4096, TOF);
-    fwrite(copychar, how, 1, RTOF);
- } while ( how!=0 );
+    do {
+       how=fread(copychar, 1, 4096, TOF);
+       fwrite(copychar, how, 1, RTOF);
+    } while ( how!=0 );
 
- free(copychar);
+    free(copychar);
+  } 
 
 /* End of copy */
    
@@ -1284,8 +1304,6 @@ else {
 
   if (quiet==FALSE) show_mode(DECRYPT,algorithm,mode,tofile,bit7_flag);
 
-
- 
   switch (mode) {
     case CBC:
       td = init_mcrypt_cbc(algorithm,keyword,len);
@@ -1325,7 +1343,7 @@ switch (mode) {
   /* decryption starts here */
    fread( ciphertext, 1, blocksize, FROMF); /* Read the IV */
    mdecrypt_cbc( td, ciphertext, blocksize);
-   
+     
   /* decryption starts here */
    do
    {
@@ -1341,9 +1359,10 @@ switch (mode) {
              }
              else { /* eof */
                ch=1;
-               if (i==0) { err_crit (_("Incorrect file type\n")); return 1;}
-               memmove(&counter, ciphertext, sizeof(counter));
-               if (counter >= blocksize) { err_crit (_("Incorrect file type\n")); return 1;}
+
+                if (i==0) {err_crit (_("Incorrect file type\n")); return 1;}
+                memmove(&counter, ciphertext, sizeof(counter));
+                if (counter >= blocksize) { err_crit (_("Incorrect file type\n")); return 1;}
              }
            }
 
@@ -1402,7 +1421,7 @@ switch (mode) {
                ch=1;
                if (i==0) {err_crit (_("Incorrect file type\n")); return 1;}
                memmove(&counter, ciphertext, sizeof(counter));
-               if (counter >= blocksize) {err_crit (_("Incorrect file type\n")); return 1;}
+               if (counter >= blocksize) { err_crit (_("Incorrect file type\n")); return 1;}
              }
            }
 
@@ -1598,24 +1617,21 @@ switch (mode) {
 
 }
 
+
 
 int check_mode(char *chain) {
-char *x, *z;
+char *x;
 int i;
 
- for (i=0;i<MAX_MODE+1;i++) {
+ for (i=0;i<10;i++) {
+  if (is_ok_mode(i)==0) {
    x=get_modes_name(i);
-   if (strlen(chain) < strlen(x)) {
-     z=chain;
-   }
-   else {
-     z=x;
-   }
-   if (strncasecmp( x, chain, strlen(z)) == 0) {
+   if (strcasecmp( x, chain) == 0) {
     free(x);
     return i;
    }
    free(x);
+  }
  }
 
 return -1;
@@ -1628,10 +1644,10 @@ char *y;
 
 
 
- for (i=0;i<=MAX_ALGO;i++) {
-   y=get_algorithms_name(i);
-   if (y!=NULL) {
-     if ( strncasecmp( y, chain, strlen(y)) == 0)  {
+ for (i=0;i<255;i++) {
+   if (is_ok_algorithm(i)==0) {
+     y=get_algorithms_name(i);
+     if ( strcasecmp( y, chain) == 0)  {
       free(y);
       return i;
      }
@@ -1641,3 +1657,360 @@ char *y;
 
 return -1;
 }
+
+
+
+/* Solaris DES Encryption */
+
+int encrypt_desc(char *fromfile, char *tofile, char *key)
+{
+unsigned int len;
+char keyword[8]; 
+char *tmp;
+FILE *TOF;
+FILE *FROMF;
+char* command=NULL;
+int cnt,c,i, td;
+char work[8];
+char iv[8];
+
+/* Crypt does not care about these 
+ * It's purpose is to be compatible with the unix crypt which is not safe.
+ * It isn't a block algorithm after all.
+ */
+  bit7_flag=BITS8;
+
+  
+/* open files */
+  if (stream_flag==TRUE) {
+	FROMF=stdin;
+	TOF=stdout;
+  }
+  else {
+#ifdef ZIP
+    if (gzipflag==FALSE && bzipflag==FALSE) {
+#endif
+	FROMF=fopen(fromfile,"rb");
+	if (FROMF==NULL) {perror("fopen");return 1;}
+	if (read_lock( fileno((FILE*)FROMF))==-1) return 1;
+	
+	TOF=fopen(tofile,"wb");
+	if (TOF==NULL) {perror("fopen");return -1;}
+	if (write_lock( fileno((FILE*)TOF))==-1) return -1;
+#ifdef ZIP
+    }
+    else {  
+           /* command if using gzip */
+        if (gzipflag==TRUE) {
+          command=malloc(strlen(GZIP)+strlen(fromfile)+5);
+          strcpy(command, GZIP);
+        }
+        if (bzipflag==TRUE) {
+          command=malloc(strlen(BZIP2)+strlen(fromfile)+5);
+          strcpy(command, BZIP2);
+        }
+        strcat(command, " -c ");
+        strcat(command, fromfile);
+
+    	FROMF=popen(command ,"r");
+      /* Waitpid is called here because popen does not return
+         NULL if the execution of the program fails.
+         thanx to Vasilis Vasaitis <vvas@hal.csd.auth.gr>
+       */
+
+#ifdef HAVE_WAITPID
+	if (FROMF==NULL || waitpid(-1, NULL, WNOHANG)==0) {perror("popen");return 1;}
+#else
+	if (FROMF==NULL) {perror("popen");return 1;}
+#endif	
+	
+	TOF=fopen(tofile,"wb");
+	if (TOF==NULL) {perror("fopen");return -1;}
+	if (write_lock( fileno((FILE*)TOF))==-1) return -1;
+    
+    }
+#endif /* ZIP */
+  }
+
+         Bzero(keyword,sizeof(keyword));
+
+	if (key==NULL) 	{
+          if (stream_flag==FALSE) {
+            sprintf(tmperr, _("File: %s\n"),fromfile);
+            err_warn (tmperr);
+          }
+          tmp=get_password(8, bit7_flag, DECRYPT, type, &len);
+          if (tmp==NULL) return 1;
+          memmove(keyword, tmp, len);
+          Bzero(tmp, strlen(tmp));
+	}
+	else {
+           if (strlen(key) > 8) {
+             key[8]='\0'; 
+             err_warn (_("Warning: Reducing the size of the key\n"));
+           }
+	   memmove(keyword, key, strlen(key)); /* Copy the 64 bits in key */
+	}
+
+       if (quiet==FALSE) show_mode(ENCRYPT,type,mode,tofile,bit7_flag); 
+
+     if (hexflag==FALSE) {
+       for(cnt = 0; cnt < 8; cnt++) {
+           c = 0;
+           for(i=0;i<7;i++)
+             if(keyword[cnt] & (1 << i))
+               c++;
+             if((c & 1) == 0) 
+               keyword[cnt] |= 0x80;
+             else
+               keyword[cnt] &= ~0x80;
+       }
+     }
+
+  switch (mode) {
+    case ECB:
+
+        td = init_mcrypt_ecb(DES,keyword, get_key_size(DES));
+
+	for(;;){
+		if((cnt = fread(work,1,8,FROMF)) != 8){
+			/* Put residual byte count in the last block.
+			 * Note that garbage is left in the other bytes,
+			 * if any; this is a feature, not a bug, since it'll
+			 * be stripped out at decrypt time.
+			 */
+			work[7] = cnt;
+		}
+
+		mcrypt_ecb(td, work, 8);	/* Encrypt block */
+
+		fwrite(work,1,8,TOF);
+		if(cnt != 8)
+			break;
+	}
+        end_mcrypt_ecb(td);
+        break;
+        
+        
+    case CBC:
+     td = init_mcrypt_cbc(DES,keyword, get_key_size(DES));
+     Bzero(iv, 8);
+     
+	for(;;){
+
+		if((cnt = fread(work,1,8,FROMF)) != 8){
+			/* Put residual byte count in the last block.
+			 * Note that garbage is left in the other bytes,
+			 * if any; this is a feature, not a bug, since it'll
+			 * be stripped out at decrypt time.
+			 */
+			work[7] = cnt;
+		}
+
+				
+		mcrypt_cbc(td,work, 8);	/* Encrypt block */
+
+		memcpy(iv,work,8);
+		fwrite(work,1,8,TOF);
+		if(cnt != 8)
+			break;
+	}
+	end_mcrypt_cbc(td);
+        
+     break;
+   default:
+     return 1;
+     break;
+  }
+                        
+
+  fclose(FROMF);
+  fclose(TOF);
+  Bzero (keyword, get_key_size(DES));
+  return 0; 	
+}
+
+
+
+/* Solaris DES Decryption */
+
+int decrypt_desc(char *fromfile, char *tofile, char *key)
+{
+unsigned int len;
+char keyword[8]; 
+char *tmp;
+FILE *TOF;
+FILE *FROMF;
+char* command=NULL;
+int cnt,c,i, td;
+char work[8],nwork[8];
+char iv[8];
+
+/* Crypt does not care about these 
+ * It's purpose is to be compatible with the unix crypt which is not safe.
+ * It isn't a block algorithm after all.
+ */
+  bit7_flag=BITS8;
+
+  
+/* open files */
+  if (stream_flag==TRUE) {
+	FROMF=stdin;
+	TOF=stdout;
+  }
+  else {
+#ifdef ZIP
+    if (gzipflag==FALSE && bzipflag==FALSE) {
+#endif
+	FROMF=fopen(fromfile,"rb");
+	if (FROMF==NULL) {perror("fopen");return 1;}
+	if (read_lock( fileno((FILE*)FROMF))==-1) return 1;
+	
+	TOF=fopen(tofile,"wb");
+	if (TOF==NULL) {perror("fopen");return -1;}
+	if (write_lock( fileno((FILE*)TOF))==-1) return -1;
+#ifdef ZIP
+    }
+    else {  
+           /* command if using gzip */
+        if (gzipflag==TRUE) {
+          command=malloc(strlen(GZIP)+strlen(fromfile)+5);
+          strcpy(command, GZIP);
+        }
+        if (bzipflag==TRUE) {
+          command=malloc(strlen(BZIP2)+strlen(fromfile)+5);
+          strcpy(command, BZIP2);
+        }
+        strcat(command, " -c ");
+        strcat(command, fromfile);
+
+    	FROMF=popen(command ,"r");
+      /* Waitpid is called here because popen does not return
+         NULL if the execution of the program fails.
+         thanx to Vasilis Vasaitis <vvas@hal.csd.auth.gr>
+       */
+
+#ifdef HAVE_WAITPID
+	if (FROMF==NULL || waitpid(-1, NULL, WNOHANG)==0) {perror("popen");return 1;}
+#else
+	if (FROMF==NULL) {perror("popen");return 1;}
+#endif	
+	
+	TOF=fopen(tofile,"wb");
+	if (TOF==NULL) {perror("fopen");return -1;}
+	if (write_lock( fileno((FILE*)TOF))==-1) return -1;
+    
+    }
+#endif
+  }
+
+         Bzero(keyword,sizeof(keyword));
+
+	if (key==NULL) 	{
+          if (stream_flag==FALSE) {
+            sprintf(tmperr, _("File: %s\n"),fromfile);
+            err_warn (tmperr);
+          }
+          tmp=get_password(8, bit7_flag, DECRYPT, type, &len);
+          if (tmp==NULL) return 1;
+          memmove(keyword, tmp, len);
+          Bzero(tmp, strlen(tmp));
+	}
+	else {
+           if (strlen(key) > 8) {
+             key[8]='\0'; 
+             err_warn (_("Warning: Reducing the size of the key\n"));
+           }
+	   memmove(keyword, key, strlen(key)); /* Copy the 64 bits in key */
+	}
+
+
+if (quiet==FALSE) show_mode(ENCRYPT,type,mode,tofile,bit7_flag); 
+
+      if (hexflag==FALSE) {
+       for(cnt = 0; cnt < 8; cnt++){
+           c = 0;
+           for(i=0;i<7;i++) {
+             if(keyword[cnt] & (1 << i))
+               c++;
+           }
+           if((c & 1) == 0)
+             keyword[cnt] |= 0x80;
+           else
+             keyword[cnt] &= ~0x80;
+       }
+      }
+      
+  switch (mode) {
+    case ECB:
+     td = init_mcrypt_ecb(DES,keyword, get_key_size(DES));
+
+	cnt = fread(work,1,8,FROMF);	/* Prime the pump */
+	for(;;){
+		mdecrypt_ecb(td,work, 8);
+
+		/* Save buffer pending next read */
+		memcpy(nwork,work,8);
+
+		/* Try to read next block */
+		cnt = fread(work,1,8,FROMF);
+		if(cnt != 8){	/* Can "only" be 0 if not 8 */
+			/* Prev block was last one, write appropriate number
+			 * of bytes
+			 */
+			cnt = nwork[7];
+			if(cnt < 0 || cnt > 7){
+				fprintf(stderr,_("Corrupted file or wrong key\n"));
+				return 1;
+			} else if(cnt != 0)
+				fwrite(nwork,1,cnt,TOF);
+			return 0;
+		} else {
+			/* Now okay to write previous buffer */
+			fwrite(nwork,1,8,TOF);
+		}
+
+	}
+        end_mcrypt_ecb(td);
+
+     break;
+    case CBC:
+
+     td = init_mcrypt_cbc(DES,keyword, get_key_size(DES));
+        Bzero(iv, 8);
+        cnt = fread(work,1,8,FROMF);	/* Prime the pump */
+	for(;;){
+		mdecrypt_cbc(td,work, 8);
+
+		/* Save buffer pending next read */
+		memmove(nwork,work,8);
+		/* Try to read next block */
+		cnt = fread(work,1,8,FROMF);
+		if(cnt != 8){	/* Can "only" be 0 if not 8 */
+			/* Prev block was last one, write appropriate number
+			 * of bytes
+			 */
+			cnt = nwork[7];
+			if(cnt < 0 || cnt > 7){
+				fprintf(stderr,_("Corrupted file or wrong key\n"));
+				return 1;
+			} else if(cnt != 0)
+				fwrite(nwork,1,cnt,TOF);
+			exit(0);
+		} else {
+			/* Now okay to write previous buffer */
+			fwrite(nwork,1,8,TOF);
+		}
+	}
+        end_mcrypt_cbc(td);
+        break;
+  }      
+
+  fclose(FROMF);
+  fclose(TOF);
+  Bzero (keyword, get_key_size(DES));
+  return 0;
+  
+}
+
+
