@@ -13,3 +13,7 @@ int check_algo(char *chain); /* input is the algorithms name... output the
                               * algorithm's number
                               */
 int check_mode(char *chain);
+
+/* Solaris des(3) */
+int encrypt_desc(char *fromfile, char *tofile, char *key);
+int decrypt_desc(char *fromfile, char *tofile, char *key);
