@@ -7,15 +7,15 @@
 main() {
 
 int td,i,j,len;
-unsigned char keyword[]="NOT_A_VERY_BIG_ONE_KEYWORD";
-unsigned char plaintext[]="A_VERY_LARGE_BLOCK"; 
+unsigned char keyword[]="                ";
+unsigned char plaintext[]="        "; 
 unsigned char ciphertext[20];
 char *name;
 
 
-  for (i=0;i<=10;i++) {
+  for (i=0;i<=11;i++) {
 
-   td = init_mcrypt_ecb(i, keyword, strlen(keyword));
+   td = init_mcrypt_ecb(i, keyword, sizeof(keyword));
    if (td<0) continue; 
 
    name=get_algorithms_name(i);
@@ -32,11 +32,11 @@ char *name;
    }
    
 
-   if (strlen(keyword)>get_key_size(i)) {
+   if (sizeof(keyword)>get_key_size(i)) {
     len=get_key_size(i);
    }
    else {
-    len=strlen(keyword);
+    len=sizeof(keyword);
    }
    
    printf ("\nkeyword:    ");
